
/************************************************************************/
/* Copyright (c) 2016 - 2022 Marnix van den Bos.                        */
/*                                                                      */
/* <marnix@xvan.nl>                                                     */
/*                                                                      */
/* This file is part of XVAN, an Interactive Fiction authoring system.  */
/*                                                                      */
/* XVAN is free software: you can redistribute it and/or modify         */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, version 3.                             */
/*                                                                      */
/* XVAN is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with XVAN.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                      */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>   /* malloc()  */
#include <string.h>
#include <ctype.h>    /* toupper() */
#include <stdint.h>

#include "keyword.h"
#include "typedefs.h"
#include "ENG-article.h"

/*************************/
/* Function declarations */
/*************************/

void ENG_PrintArticle(sysDescr*, int32_t);

/************************/
/* Function definitions */
/************************/

void ENG_PrintArticle(sysDescr *descr, int32_t type)
{
  int       i   = 0;
  int32_t   id  = NO_ID;
  wordTable *wt = word_table;

  if (descr->article != NO_ID) {
    /* there is an article, so check if we */
    /* must print something                */
    switch (article) {
      case ARTICLE_THE:
        /* clear article or else the white space will */
        /* not be printed by PrintString()            */
        article = 0;
        PrintWord(descr->article);
        PrintString(" ");
        break;
      case ARTICLE_A:
        /* We must print either 'a' or 'an'.             */
        /* Check if the next word starts with a vowel.   */
        /* Next word is either an adjective or the noun. */
        if ((i = descr->nr_of_adjectives-1) == -1) {
          /* there are no adjectives, we must have the noun */
          id = descr->noun;
          /* set i to 0 for indexing wt in next while loop */
          i = 0;
        }
        else
          id = descr->adjectives[i];

        /* Find word with id. */
        while (i<nr_of_words && wt[i].id != id)
          i++;
        if (i == nr_of_words) {
          /* Internal error; this shouldn't happen. */
          PrintError(50, &((resultStruct) {VALUE, NONE, id}), NULL);
        }
        else {
          /* clear article or else the white space will */
          /* not be printed by PrintString()            */
          article = 0;
          if (IsVowel( (wt[i].print_word)[0]))
            PrintString("an ");
          else
            PrintString("a ");
        }
        break;
      default:
        /* There is an article in the system description,  */
        /* but no %a or %the in the string, so don't print */
        /* the article.                                    */
        break;
    } /* switch */
  }
  else {
    /* There is no article to print. Set article */
    /*to 0 or else spaces will be ignored.        */
    article = 0;
  }
}
