
/************************************************************************/
/* Copyright (c) 2016 - 2022 Marnix van den Bos.                        */
/*                                                                      */
/* <marnix@xvan.nl>                                                     */
/*                                                                      */
/* This file is part of XVAN, an Interactive Fiction authoring system.  */
/*                                                                      */
/* XVAN is free software: you can redistribute it and/or modify         */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, version 3.                             */
/*                                                                      */
/* XVAN is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with XVAN.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                      */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "keyword.h"
#include "typedefs.h"
#include "ENG-choices.h"

/*************************/
/* Function declarations */
/*************************/

void ENG_GetChoice(char[]);

/************************/
/* Function definitions */
/************************/

void ENG_GetChoice(char response_text[])
{
  int  i     = 0;
  int  cont  = 1;
  int  choice = 0;
  char text_to_print[OUTPUT_LINE_LEN];
  char line_buf[INPUT_LINE_LEN];

  PrintString("\nWhat do you want to do next?\n");

  while (i<MAX_CHOICES && cont) {
    if (choices[i].choice != NULL) {
      snprintf(text_to_print, OUTPUT_LINE_LEN, "   %d. %s\n", i+1, choices[i].choice);
      PrintString(text_to_print);
      i++;
    }
    else {
      cont = 0;
    }
  }
  /* at the end of this loop, the value of i will */
  /* be 1 more than the number of choices.        */

  sprintf(text_to_print, "\nEnter a number from 1 to %d: ", i);
  PrintString(text_to_print);

  cont = 1;
  while (cont) {
    /* now get the choice number */
    Output(outputbuffer);

    GetGlkInput(line_buf);

    /* convert to a number */
    if (StringToNum(line_buf, &choice)) {
      /* valid number syntax */
      if (choice > 0 && choice < i+1) {
        cont = 0;
      }
    }
    if (cont) {
      sprintf(text_to_print, "\nEnter a number between 1 and %d: ", i);
      PrintString(text_to_print);
    }
  }

  /* use INPUT_LINE_LEN because response_text actually is line_buf */
  strncpy(response_text, choices[choice-1].response, INPUT_LINE_LEN);
  response_text[INPUT_LINE_LEN-1] = '\0';  /* @!@ */
  PrintString("\n");

/* @!@ */ /* dubbele tekst verwijderd */

  return;
}
