
/************************************************************************/
/* Copyright (c) 2016 - 2022 Marnix van den Bos.                        */
/*                                                                      */
/* <marnix@xvan.nl>                                                     */
/*                                                                      */
/* This file is part of XVAN, an Interactive Fiction authoring system.  */
/*                                                                      */
/* XVAN is free software: you can redistribute it and/or modify         */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, version 3.                             */
/*                                                                      */
/* XVAN is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with XVAN.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                      */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>   /* malloc() */
#include <string.h>
#include <stdint.h>

#include "keyword.h"
#include "typedefs.h"
#include "trnslate.h"

#include "ENG-moreinfo.h"

/*************************/
/* Function declarations */
/*************************/

void ENG_MoreInfo(extendedSysDescr*, match*, char*);
void ENG_PrintNotFound(extendedSysDescr*);

/************************/
/* Function definitions */
/************************/

void ENG_MoreInfo(extendedSysDescr *descr, match *hits, char *line_buf)
{
  extendedSysDescr *sd;
  int32_t      i =0;

  /* do not print a capital */
  capital = 0;

  outputbuffer = ResetString(outputbuffer);

  if ((descr->part1).noun == NO_ID) {
    PrintString("What ");
  }
  else {
    PrintString("Which ");
    PrintExtendedSysDescr(descr);
  }
  PrintString(" do you mean?\nThe ");

  for (i=0; i<hits->nr_of_hits; i++) {
    if (i != 0 && i != hits->nr_of_hits-1) {
      PrintString(", the ");
    }
    else if (i == hits->nr_of_hits-1) {
      PrintString(" or the ");
    }
    if (IsLocId(hits->matched_objs[i]))
      sd = loc_dir[hits->matched_objs[i]-FIRST_LOCATION_ID].descr;
    else
      /* It must be an object; forget the test. */
      sd = obj_dir[hits->matched_objs[i]-FIRST_OBJECT_ID].descr;
    PrintExtendedSysDescr(sd);
  }

  PrintString("?\n");
  Output(outputbuffer);

  do {
    GetInput(line_buf, prompt);
  } while (line_buf[0] == PLAYER_COMMENT);
}


void ENG_PrintNotFound(extendedSysDescr *descr)
{
  outputbuffer = ResetString(outputbuffer);
  if (actor == PLAYER) {
    PrintString("You don't ");
  }
  else {
    PrintId(actor);
    PrintString(" doesn't ");
  }
  PrintString("see that here.\n");
  Output(outputbuffer);
}
