
/************************************************************************/
/* Copyright (c) 2016 - 2022 Marnix van den Bos.                        */
/*                                                                      */
/* <marnix@xvan.nl>                                                     */
/*                                                                      */
/* This file is part of XVAN, an Interactive Fiction authoring system.  */
/*                                                                      */
/* XVAN is free software: you can redistribute it and/or modify         */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation, version 3.                             */
/*                                                                      */
/* XVAN is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* You should have received a copy of the GNU General Public License    */
/* along with XVAN.  If not, see <http://www.gnu.org/licenses/>.        */
/*                                                                      */
/************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>

#include "keyword.h"
#include "typedefs.h"
#include "choices.h"

/*************************/
/* Function declarations */
/*************************/

void    GetChoice(char[]);
int32_t UpdateChoicesMenu(void);

/************************/
/* Function definitions */
/************************/

void GetChoice(char response_text[])
{
  switch(story_info.story_language) {
    case NL:
      NL_GetChoice(response_text);
      break;
    case ENG:
      ENG_GetChoice(response_text);
      break;
    default:
      /* unknown language, use English */
      ENG_GetChoice(response_text);
      break;
  }
}


int32_t UpdateChoicesMenu()
{
  /* this function is called in choice or hybrid mode   */
  /* it calls t_choice for all locations and objects in */
  /* scope and builds a choice list that is send to the */
  /* front-end                                          */
  /* next, it catches the player's choice and returns   */
  /* it for processing as a normally entered command    */


  int          i          = 0;
  int32_t     *cont_list  = NULL;
  int          size       = nr_of_locs+nr_of_objs;
  resultStruct result     = {OK, NONE, OK};
  usrActionRec dummy_rec;

  /* int32_t cont_list[nr_of_locs+nr_objs] does not work, so we use malloc() */
  if ( (cont_list = (int32_t *) malloc((nr_of_locs+nr_of_objs)*sizeof(int32_t))) == NULL) {
    PrintError(15, NULL, "UpdateChoices()");
    return(QUIT);
  }

  /* we need dummy_rec to enforce an error when t_choice */
  /* ends up in XeqVerbDefault()                         */
  InitUsrActionRec(&dummy_rec);   /* @!@ */

  /* determine the set of locations/objects in scope */
  /* unused items in list[] will have value NO_ID    */
  ContList(curr_loc, cont_list, CURR_LOC);

  /* now execute t_choice for each list member      */
  /* each member may add options to the choice list */
  while (i<size && cont_list[i] != NO_ID) {
    result = XeqTrigger(cont_list[i], CHOICE, &dummy_rec, 0);
    switch (result.tag) {
      case QUIT:
        free(cont_list);
        return(QUIT);
        break;
      case DISAGREE:
        /* disagree for a t_choice means ok for the caller */
        free(cont_list);
        return(OK);
        break;
      default:
        /* do nothing */
        break;
    }
    i++;
  }

  Output(outputbuffer);
  /* ok, the choice-triggers printed a list of one or more choices */

  free(cont_list);
  return(OK);
}
